import fastapi
from fastapi import Response
import requests
import logging
import conf
from models.info import InfoOut, ServiceDetail, Services
import _version as b
from starlette_context import context
from pydantic import BaseModel
import correlation

from starlette.status import HTTP_400_BAD_REQUEST, HTTP_503_SERVICE_UNAVAILABLE

router = fastapi.APIRouter()


class Detail(BaseModel):
    detail: str


logger = logging.getLogger(__name__)


@router.get(
    "/info",
    response_model=InfoOut,
    responses={
        HTTP_400_BAD_REQUEST: {"model": Detail},
        HTTP_503_SERVICE_UNAVAILABLE: {"model": Detail},
    },
)
def return_version_info(response: Response):
    """
    Return Service version information.
    Expected returned JSON is in "InfoOut" schema, which include Services and ServiceDetail schemas.

    ### Permission required to use this API:

        None
    """
    logging.setLogRecordFactory(
        correlation.set_correlation_id(context["correlation_id"])
    )
    logger = logging.getLogger(__name__)
    response.headers["X-Correlation-ID"] = context["correlation_id"]

    url = conf.OPA_CONFIG_API
    opa_version = "unknown"
    logger.debug("debug message")
    logging.warning("warning message")
    logger.info(f"Log level is {logger.getEffectiveLevel()}")
    try:
        r = requests.get(url, timeout=10)
        if "result" in r.text:
            data = r.json()
            opa_version = data["result"]["labels"]["version"]
    except requests.exceptions.HTTPError as e:
        logging.warning(f"Http Error to OPA {url}: {e}")
    except requests.exceptions.ConnectionError as e:
        logging.warning(f"Error Connecting to OPA {url}: {e}")
    except requests.exceptions.Timeout as e:
        logging.warning(f"Timeout Error to OPA {url}: {e}")
    except requests.exceptions.RequestException as e:
        logging.warning(f"Connection error to OPA {url}: {e}")

    svcdetail = ServiceDetail(version=opa_version)
    opa = Services(opa=svcdetail)
    info = InfoOut(
        version=b.__version__,
        artifactId=None,
        name="policy",
        groupId="org.opengroup.osdu",
        buildTime=b.__committimestamp__,
        branch=b.__branch__,
        commitId=b.__commitid__,
        commitMessage=b.__commitmessage__,
        connectedOuterServices=opa,
    )
    return info
