# Policy Service
os-policy-service is a service that provides a set of APIs for management and evaluation of dynamic policies in OSDU.

These instructions will get you a copy of the project up and running on your local machine for development and testing purposes.

### Prerequisites

* Python3
* [OPA](https://www.openpolicyagent.org/docs/latest/#1-download-opa)
* OSDU Instance deployed on AWS

## Run Locally

### Run OPA server
- The policy service will call OPA. So first run OPA server in the OPA installation folder. By default, OPA listens for normal HTTP connections on 0.0.0.0:8181.
```
./opa run --server
```

## Testing
 
 ### Running Integration Tests 
 This section describes how to run OSDU Integration tests
 
 You will need to have the following environment variables defined.
 
| name | example value | description | sensitive? |
| ---  | ---   | ---         | ---        |
| `AWS_REGION` | `us-east-1` | yes | The region where resources needed by the service are deployed | no |
| `ENTITLEMENTS_BASE_URL` | `http://localhost:8081` or `https://some-hosted-url` | Specify the base url for an entitlements service instance. Can be run locally or remotely | no |
| `ENTITLEMENTS_BASE_PATH` | `/api/entitlements/v2/groups` | Specify the base path for an entitlements service instance | no |
| `LEGAL_BASE_URL` | `http://localhost:8082` or `https://some-hosted-url` | Specify the base url for an legal service instance. Can be run locally or remotely | no |
| `AWS_DEPLOYMENTS_SUBDIR` | `deployment/scripts/aws` | The subdirectory for AWS deployment 
| `AWS_COGNITO_USER_POOL_ID` | `us-east-1_xxxxxxxx` | User Pool Id for the reference cognito | no |
| `AWS_COGNITO_CLIENT_ID` | `xxxxxxxxxxxx` | Client ID for the Auth Flow integrated with the Cognito User Pool | no |
| `AWS_COGNITO_AUTH_FLOW` | `USER_PASSWORD_AUTH` | Auth flow used by reference cognito deployment | no |
| `AWS_COGNITO_AUTH_PARAMS_USER` | `int-test-user@testing.com` | Int Test Username | no |
| `AWS_COGNITO_AUTH_PARAMS_PASSWORD` | `some-secure-password` | Int Test User/NoAccessUser Password | yes |
| `AWS_COGNITO_AUTH_PARAMS_USER_NO_ACCESS` | `no-access-user@testing.com` | Int Test No Access Username | no |
| `AWS_BASE_URL` | `http://localhost:8080` | Specify the base url where the policy service is hosted (this can be found in server.py) | no | 

 **Creating a new user to use for integration tests**
 ```
 aws cognito-idp admin-create-user --user-pool-id ${AWS_COGNITO_USER_POOL_ID} --username ${AWS_COGNITO_AUTH_PARAMS_USER} --user-attributes Name=email,Value=${AWS_COGNITO_AUTH_PARAMS_USER} Name=email_verified,Value=True --message-action SUPPRESS

 aws cognito-idp initiate-auth --auth-flow ${AWS_COGNITO_AUTH_FLOW} --client-id ${AWS_COGNITO_CLIENT_ID} --auth-parameters USERNAME=${AWS_COGNITO_AUTH_PARAMS_USER},PASSWORD=${AWS_COGNITO_AUTH_PARAMS_PASSWORD}
 ```
 
 **Entitlements group configuration for integration accounts**
 <br/>
 In order to add user entitlements, run entitlements bootstrap scripts in the entitlements project
 
| AWS_COGNITO_AUTH_PARAMS_USER | AWS_COGNITO_AUTH_PARAMS_USER_NO_ACCESS | 
|----| --- |
| service.entitlements.user | service.entitlements.user |
| service.legal.user |
| service.policy.user |
| service.policy.admin |
 
 Execute following command to build code and run all the integration tests:

#### Run Tests Simulating Pipeline

* Prior to running tests, scripts must be executed locally to generate pipeline env vars

```bash
# Set neccessary environment variables here as defined in the table above

deployment/scripts/aws/bootstrap.sh

tests/aws/build-aws/prepare-dist.sh

tests/aws/build-aws/run-tests.sh
```



## License
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
[http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)
 
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
