# test_integration_011_utility.py
import pytest

# requires pytest_dependency to be installed
import unittest
import responses
from fastapi.testclient import TestClient
import requests
import re
import logging
import sys
import os
import base64
import json
from pathlib import Path

# so the directly in which you run pytest or unittest doesn't matter as much
sys.path.append(os.path.abspath(".."))
sys.path.append("tests")

# local
from main import app
from auth import auth
import conf
import testlib

# override dependency injection for authentication to entitlement service
from override_depends import (
    override_require_authorized_user,
    set_authorize_session,
    ADMIN_ONLY_SVC,
    USER_AND_ADMIN_SVC,
    USER_ONLY_SVC,
    OTHER_ONLY_SVC,
)

TEST_DATA_DIR = Path(__file__).resolve().parent / "data"

client = TestClient(app)


@pytest.mark.dependency(name="require_token")
def test_require_token(token):
    assert token is not None, "No token provided on command line"


def test_cloud_provider(cloud_provider):
    """Check CLOUD_PROVIDER"""
    assert cloud_provider is not None, "No CLOUD_PROVIDER env set"


@pytest.mark.dependency(name="require_token")
def test_validate_with_template(token, data_partition, service_url, cloud_provider):
    """
    Test Validate API with template function
    """
    policy_id = "dataauthz.rego"
    data = """package osdu.partition["${DATA_PARTITION}"].${name}

import data.osdu.instance.dataauthz as centralauthz

records := centralauthz.records"""

    bdata = data.encode("utf-8")
    files = {"file": (policy_id, bdata)}
    url = service_url + conf.SERVICE_BASE_PATH + f"/validate/{policy_id}"
    params = {"template": True}
    headers = {"Authorization": "Bearer " + token, "data-partition-id": data_partition}
    r = requests.put(url, headers=headers, params=params, files=files)
    if r.status_code != 200:
        pytest.xfail(
            f"service_url /validate fail for {cloud_provider}: {r.status_code} {r.text} {url}"
        )


@pytest.mark.parametrize("regofile", testlib.get_rego_templates())
@pytest.mark.dependency(depends=["require_token"])
def test_validate_policies(token, data_partition, service_url, domain, regofile):
    assert token is not None, "No token provided on command line"
    testlib.put_policy_test_data(
        client=requests,
        token=token,
        data_partition=data_partition,
        service_url=service_url,
        domain=domain,
        filename=regofile,
        validate=True,
    )
