# test_integration_034_compile.py
import pytest

# requires pytest_dependency to be installed
import unittest
import responses
from fastapi.testclient import TestClient
import requests
import re
import logging
import sys
import os
import base64
import json
import time
from pathlib import Path
from string import Template

# so the directly in which you run pytest or unittest doesn't matter as much
sys.path.append(os.path.abspath(".."))
sys.path.append("tests")

# local
from main import app
from auth import auth
import conf
import testlib

# override dependency injection for authentication to entitlement service
from override_depends import (
    override_require_authorized_user,
    set_authorize_session,
    ADMIN_ONLY_SVC,
    USER_AND_ADMIN_SVC,
    USER_ONLY_SVC,
    OTHER_ONLY_SVC,
)

TEST_DATA_DIR = Path(__file__).resolve().parent / "templates"

client = TestClient(app)


@pytest.mark.dependency(name="require_token")
def test_require_token(token):
    assert token is not None, "No token provided on command line"


def post_request(
    token, data_partition, files, metrics=False, instrument=False, service_url=False
):
    url = conf.SERVICE_BASE_PATH + "/compile"
    params = {"metrics": metrics, "instrument": instrument}
    if service_url:
        r = requests.post(
            service_url + url,
            files=files,
            params=params,
            headers={
                "Authorization": "Bearer " + token,
                "data-partition-id": data_partition,
            },
        )
    else:
        r = client.post(
            url,
            files=files,
            params=params,
            headers={
                "Authorization": "Bearer " + token,
                "data-partition-id": data_partition,
            },
        )

    assert r.status_code == 200, f"Expected to get 200 from {url} {r.text}"
    assert "result" in r.text, f"Expected to get 'result' from {url} {r.text}"
    assert "queries" in r.text, f"Expected to get 'queries' from {url} {r.text}"
    result = r.json()


@pytest.mark.opa_access
@pytest.mark.dependency(depends=["require_token"])
def test_compile(token, data_partition):
    assert token is not None, "No token provided on command line"
    assert data_partition is not None, "No data partition provided on command line"

    num_tests = 0
    p = os.path.dirname(os.path.abspath(__file__))
    template_datadir = os.path.join(p, "..", "templates")
    for filename in sorted(os.listdir(template_datadir)):
        filetype = ".json"
        file_path = os.path.join(template_datadir, filename)
        # checking if it is a file
        if os.path.isfile(file_path):
            if file_path.endswith(filetype) and filename.startswith(
                "compile"
            ):  # compile*.json
                input_json = os.path.basename(file_path)
                print(input_json)
                print(f"file: {file_path}, input: {input_json}")
                f = open(file_path, "r")
                data = f.read()
                f.close()
                template_data = Template(data)
                data = template_data.substitute({"data_partition": data_partition})
                bdata = data.encode("utf-8")
                files = {"file": (input_json, bdata)}

                post_request(token=token, data_partition=data_partition, files=files)


@pytest.mark.dependency(depends=["require_token"])
def test_compile_diag_service_url(token, data_partition, service_url):
    assert token is not None, "No token provided on command line"
    assert data_partition is not None, "No data partition provided on command line"

    num_tests = 0
    p = os.path.dirname(os.path.abspath(__file__))
    template_datadir = os.path.join(p, "..", "templates")
    for filename in sorted(os.listdir(template_datadir)):
        filetype = ".json"
        file_path = os.path.join(template_datadir, filename)
        # checking if it is a file
        if os.path.isfile(file_path):
            if file_path.endswith(filetype) and filename.startswith(
                "compile"
            ):  # compile*.json
                input_json = os.path.basename(file_path)
                print(input_json)
                print(f"file: {file_path}, input: {input_json}")
                f = open(file_path, "r")
                data = f.read()
                f.close()
                template_data = Template(data)
                data = template_data.substitute({"data_partition": data_partition})
                bdata = data.encode("utf-8")
                files = {"file": (input_json, bdata)}

                post_request(
                    token=token,
                    data_partition=data_partition,
                    files=files,
                    metrics=True,
                    instrument=True,
                    service_url=service_url,
                )
