#!/usr/bin/env bash
# USAGE:
# -c CLOUD_PROVIDER
# Expects DATA_PARTITION 
# If this test is running with a different DATA_PARTIION value then policy service
# then provide data partition of policy service as TARGET_DATA_PARTITION for pathing of policies

while getopts c: flag
do
    case "${flag}" in
        c) CLOUD_PROVIDER=${OPTARG};;
    esac
done

echo "#### run-load-tests ####"
cat /etc/os-release
python3 --version
python3 -m venv env
source env/bin/activate
echo "#### Installing ####"
python3 -m pip install --upgrade pip
python3 -m pip install -r requirements.txt

echo "#### Setting TOKEN ####"
# AWS
if [ ${CLOUD_PROVIDER} == "aws" ]; then
    export SERVICE_URL=$AWS_BASE_URL
    export TOKEN=$(python3 ../app/tests/aws/aws_jwt_client.py)
    sleep 165
elif [ ${CLOUD_PROVIDER} == "azure" ]; then
    echo "## AZURE_DATA_PARTITION = $AZURE_DATA_PARTITION"
    yum -y install curl jq
    python3 -m pip install -r requirements.txt
    export TOKEN=$(curl -ks -XPOST "https://login.microsoftonline.com/${AZURE_TENANT_ID}/oauth2/token" -d "grant_type=client_credentials&client_id=${AZURE_PRINCIPAL_ID}&client_secret=${AZURE_PRINCIPAL_SECRET}&resource=${AZURE_APP_ID}" | jq --raw-output '.access_token')
    export SERVICE_URL=https://${AZURE_DNS_NAME}
    sleep 30
elif [ ${CLOUD_PROVIDER} == "gcp" ]; then
    export SERVICE_URL=$HOST
    export TOKEN=$(python3 ../app/tests/baremetal/baremetal_jwt_client.py)
    sleep 30
elif [ ${CLOUD_PROVIDER} == "local-gcp" ]; then
    export DATA_PARTITION=osdu
    export TOKEN="$(gcloud auth print-access-token)"
    export SERVICE_URL="http://localhost:8080"
elif [ ${CLOUD_PROVIDER} == "local" ]; then
    locust --users 100 --spawn-rate 1 --run-time 10m -H http://localhost:8080
    exit
elif [ ${CLOUD_PROVIDER} == "remote" ]; then
    locust --users 100 --spawn-rate 1 --run-time 10m -H $BASE_URL
    exit
else
  echo "Not supported cloud provider ${CLOUD_PROVIDER}"
  exit 1
fi

if [ -z $TOKEN ]; then
    echo "[ERROR] Not able to get token (TOKEN)"
    exit 1
fi

if [ -z $DATA_PARTITION ]; then
    echo "[ERROR] Not able to get data partition id (DATA_PARTITION)"
    exit 1
fi

if [ -z $TARGET_DATA_PARTITION ]; then
    export TARGET_DATA_PARTITION=$DATA_PARTITION
fi

echo "#### ENV ####"
echo DATA_PARTITION: $DATA_PARTITION
echo TARGET_DATA_PARTITION: $TARGET_DATA_PARTITION
echo CLOUD_PROVIDER: $CLOUD_PROVIDER
echo SERVICE_URL: $SERVICE_URL

echo "#### Running ####"
locust --headless --users 10 --spawn-rate 1 --run-time 10m -H $SERVICE_URL --only-summary --csv=stats_${CLOUD_PROVIDER}_
